#!/usr/bin/python3

import psycopg2, subprocess, os, sys, random
import numpy as np
from random import randint 
from datetime import datetime, timedelta

host = 'localhost'
ourDB = 'cuqbank'		# change as per the DB name

'''
Ma,Mb,Mc : marks for A, B and C types
Na, Nb, Nc : Number of questions to be answered
Ta, Tb, Tc : Number of questions given  
'''
# For CU
qpdataM70CU = {'Ma':3,'Mb':6,'Mc':10,'Na':8,'Nb':6,'Nc':1,'Ta':10,'Tb':8,'Tc':2,'DUR':120}
qpdataM50CU = {'Ma':2,'Mb':6,'Mc':10,'Na':8,'Nb':4,'Nc':1,'Ta':10,'Tb':5,'Tc':2,'DUR':90}

# Kannur Univ colleges
qpdataM70KU = {'Ma':3,'Mb':6,'Mc':14,'Na':6,'Nb':4,'Nc':2,'Ta':8,'Tb':6,'Tc':3,'DUR':120}
qpdataM50KU = {'Ma':2,'Mb':6,'Mc':14,'Na':6,'Nb':4,'Nc':1,'Ta':8,'Tb':6,'Tc':2,'DUR':90}

# Models below are for KU departments only
qpdataM32KU = {'Ma':1,'Mb':3,'Mc':6, 'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM40KU = {'Ma':2,'Mb':3,'Mc':7, 'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM48KU = {'Ma':1,'Mb':5,'Mc':9, 'Na':6,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM60KU = {'Ma':3,'Mb':5,'Mc':10,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM80KU ={'Ma':4,'Mb':7,'Mc':13,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}

'''
Fields of the Course Table (Postgres RDBMS)
qtext  		VARCHAR   | Question in LaTex
atext  		VARCHAR   | Answer etc. in LaTex
author 		VARCHAR   | Author info
modulenum 	int       | module Number (Course material divided into 8 modules)
qtype 		CHAR(1)   | Question Type A: Short, B: medium, C: essay type
qnum 		int       | Question number, starts from 1, maximum is 99 (no gaps)
relations 	VARCHAR   | Specify other questions having overlap with this one
mark 		int       | Mark 
status 		VARCHAR   | medium, easy, tough or inactive.
figname 	VARCHAR   | Name of the figure, if any.
figure 		bytea     | Figure data

Each record fetched from the DB is stored into a Python list.
'''

QTEXT = 0		# indices for the fields stored in the list
QATEXT = 1
QAUTHOR = 2
QMODNUM = 3
QTYPE = 4
QSN = 5
QRELATIONS = 6
QMARK = 7
QSTAT = 8
QFIGNAME = 9
QFIG = 10
QUSED = 11    # added by this program
QPART2 = 12   # second question of C

SINGLEQL = 12	# length of the list after adding the 'used' flag

NMODS = 8   # Total course matter is divided in to 8 modules, to ensure uniformity


class Course:
	# Arrays to store the number of questions in each Part
	naq = None		
	nbq = None
	ncq = None
	# 2D Arrays to store the questions from each module
	Atypes = None
	Btypes = None
	Ctypes = None
	#Lists to store the selected questions
	AQ = []
	BQ = []
	CQ = []	
	moduleMarks = [None]*8

	def __init__(self):
		pass
		
	def findCourse(self, courseCode):
		'''
		Fetches the "coursedetails" Table from Postgres and searches for the
		specified course.
		'''
		conn = psycopg2.connect(database=ourDB, host=host, user='dummy', password="dum123")
		cursor = conn.cursor()
		cursor.execute('select * from coursedetails;')
		res = cursor.fetchall()
		found = False
		for row in res:
			print(len(row[0]), len(courseCode))
			if row[0] == courseCode:
				found = True
				self.courseCode = row[0]
				self.courseName = row[1]
				self.subject = row[2]
				self.degree = row[3]
				self.syllabusYear = row[4]
				self.semester = row[5]
				self.language = row[6]
				self.qpModel = row[7]
				self.duration = row[8]	
				s = row[9].split(':')	
				self.numModules = eval(s[0])
				for k in range(self.numModules):
					self.moduleMarks[k] = eval(s[k+1])	
				print(self.qpModel, self.numModules, self.moduleMarks)

		if found == False:
			print('Course Code not found')
			sys.exit('Error')	
		return True

	def fetchData(self):
		'''
		Fetches the question records of the specified course from the Postgres RDBMS and stores them
		to 2D(NMODS x Number of questions) arrays Atypes, Ntypes and Ctypes.
		Stores the number of questions info in arrays naq, nbq and ncq.
		'''
		try:
			conn = psycopg2.connect(database=ourDB, host=host, user='reader', password = passwd)
			cursor = conn.cursor()	
			cursor.execute("SELECT * FROM %s"%courseCode)	
			results = cursor.fetchall()
		except:
			print('Connection to Database "%s" failed. Check password.'%ourDB)
			sys.exit('Database connection failed')
		
			
		# Count the number of questions in each section, type wise
		self.naq = np.zeros(NMODS, dtype=int)
		self.nbq = np.zeros(NMODS, dtype=int)
		self.ncq = np.zeros(NMODS, dtype=int)

		for row in results:
			for k in range(NMODS):		# k goes from 0 to NMODS-1. Add 1 while comparing
				if row[QTYPE] == 'A' and row[QMODNUM] == k+1:
					self.naq[k] += 1
				if row[QTYPE] == 'B' and row[QMODNUM] == k+1:
					self.nbq[k] += 1
				if row[QTYPE] == 'C' and row[QMODNUM] == k+1:
					self.ncq[k] += 1
		
		print('NQs ',self.naq, self.nbq, self.ncq)
		# Make 3 array of objects, size NMODS, to transfer the SQL query results
		self.Atypes = np.empty(NMODS,object)
		self.Btypes = np.empty(NMODS,object)
		self.Ctypes = np.empty(NMODS,object)

		for k in range(NMODS):	# Array size = NMODS x NoQ in each (variable)
			self.Atypes[k] = np.empty(self.naq[k], object)
			self.Btypes[k] = np.empty(self.nbq[k], object)
			self.Ctypes[k] = np.empty(self.ncq[k], object)

		for k in range(len(results)):	# Copy to 3 2D Array of objects
			row = list(results[k])
			row.append(False)	# Add the 'QUSED' flag, set False			
			if row[QSTAT] == 'inactive':
				row[QUSED] = True
			if row[QSTAT] == 'easy' and qpLevel == 'tough':
				row[QUSED] = True
			if row[QSTAT] == 'tough' and qpLevel == 'easy':
				row[QUSED] = True
			if row[QTYPE] == 'A':
				sec = row[QMODNUM]
				qn = row[QSN]
				self.Atypes[sec-1][qn-1] = row	# Indices from 0 to NMODS-1 (not 1 to NMODS)
			if row[QTYPE] == 'B':
				sec = row[QMODNUM]
				qn = row[QSN]
				self.Btypes[sec-1][qn-1] = row
			if row[QTYPE] == 'C':
				sec = row[QMODNUM]
				qn = row[QSN]
				self.Ctypes[sec-1][qn-1] = row
			if row[QFIGNAME] != None:			# Save is there is a figure
				f = open(row[QFIGNAME],'wb')
				f.write(row[QFIG])
				f.close()	
		print('Fetched %d Questions'%len(results))		
		
	#--------------- Select Questions for QP ---------------------
	def markRelations(self,module,relation): # Mark the specified questions as used
		if relation == '':
			return
		rels = relation.split(',')
		for r in rels:
			r = r.strip()
			rtype = r[0]
			rval = r[1:]
			rqn = eval(r[1:])-1
			if rtype == 'C':
				self.Ctypes[module][rqn][QUSED] = True
			if rtype == 'B':
				self.Btypes[module][rqn][QUSED] = True
			if rtype == 'A':
				self.Atypes[module][rqn][QUSED] = True

	def showDist(self, ia,ib,ic):
		Ma,Mb,Mc = self.qpd['Ma'],self.qpd['Mb'],self.qpd['Mc']
		Ta,Tb,Tc = self.qpd['Ta'],self.qpd['Tb'],self.qpd['Tc']
		t = Ta*Ma +Tb*Mb +Tc*Mc
		#print(self.moduleMarks, self.pm2, )
		pat = '<h3>Report:</h3>\n'
		pat += '%s has %d Modules. Marks ['%(self.courseCode[6:],self.numModules)
		tm = 0
		for k in range(self.numModules):
			pat += '%3.1f '%self.moduleMarks[k]
			tm += self.moduleMarks[k]
		pat += '] = %d.<br>\n'%tm
		pat += 'All Questions %dx%d + %dx%d + %dx%d = %d marks. '%(Ta,Ma,Tb,Mb,Tc,Mc,t)
		pat += 'Scaling gives ['
		for k in range(self.numModules):
			pat += '%3.1f '%self.pm2[k]
		pat += ']<br><br>\n'
		pat += 'Module wise distribution of the generated paper(with the deviation):<br>\n'
		 
		mtotal = 0
		mark = ''
		for k in range(self.numModules):
			mp = 0
			pat += 'Questions from Module %d | '%(k+1)
			if k in ic:
				nc = ic.count(k)
				mp += nc * self.qpd['Mc']
				pat += '%s'%('C'*nc)
			if k in ib:
				nb = ib.count(k)
				mp += nb * self.qpd['Mb']
				pat += '%s'%('B'*nb)
			if k in ia:
				na = ia.count(k)
				mp += na * self.qpd['Ma']
				pat += '%s'%('A'*na)
			pat += ' : %d(%3.1f)<br>\n'%(mp,(mp-self.pm2[k]))
			mtotal += mp
			mark += '%d:'%mp
		#pat += ': Total %2d marks<br>\n'%mtotal
		#pat += mark
		f = open('pat.txt','w')
		f.write(pat)	
		print(pat)		

	#------------------------different selection models----------------------
	def selectM70KU(self):
		qpd = self.qpd
		Ma,Mb,Mc = qpd['Ma'],qpd['Mb'],qpd['Mc']
		Na,Nb,Nc = qpd['Na'],qpd['Nb'],qpd['Nc']
		Ta,Tb,Tc = qpd['Ta'],qpd['Tb'],qpd['Tc']
		nmods = self.numModules
		ratio = (Ta*Ma +Tb*Mb +Tc*Mc)/(Na*Ma +Nb*Mb +Nc*Mc)
		for k in range(self.numModules):
			self.moduleMarks[k] = 70/self.numModules	# Assign equal marks to all Modules
		tot = 0
		self.pm2 = []
		for k in range(self.numModules):
			x = self.moduleMarks[k]*ratio
			self.pm2.append(x)
			tot += x		
		ic = []
		ic.append(randint(0, NMODS -1))
		if ic[0] % 2:  		# odd number
			ic.append(ic[0] - 1)
		else:
			ic.append(ic[0] + 1)
		while 1:
			n = randint(0,NMODS-1)
			if n not in ic:
				ic.append(n)
				break
		ic3 = [ic[2]]   # 2 modules from which the third essay is picked
		if ic[2] % 2:  		
			ic3.append(ic[2] - 1)
		else:
			ic3.append(ic[2] + 1)			
				
		ib = []
		for k in range(NMODS):
			if k not in ic + ic3: 
				ib.append(k)
				if k%2 == 0:
					ib.append(k+randint(0,1))
		ia = []
		for k in range(NMODS):
			if k not in [ic[0], ic[1]]:
				ia.append(k)
		for k in range(0,NMODS,2):		
				if k not in ic[:2] and k not in ic3:
					ia.append(k + randint(0,1))
		for k in range(NMODS):
			if ia[k] == ic3[1]:     # move A type to the same module # of selected C
				ia[k] = ic3[0]			
		return ia,ib,ic

	def selectM50KU(self):  # For 50 marks, modules to pick A,B and C types.
		NMODS50 = 6			# Only the first Six modules
		ic = []
		ic.append(randint(0, NMODS50-1))	 # pick from 6 modules only
		if ic[0] % 2:  		# odd number
			ic.append(ic[0] - 1)
		else:
			ic.append(ic[0] + 1)

		ib = []
		for k in range(NMODS50):
			if k not in ic:
				ib.append(k)
		for k in range(0,NMODS50,2):		
				if k not in ic:
					ib.append(k + randint(0,1))
		
		ia = []
		for k in range(NMODS50):
			if k not in ic:
				ia.append(k)
				ia.append(k)
		return ia,ib,ic
		
	def selectM60KU(self):   # Ma=3, Mb=5, Mc=10 , Ta=6, Tb=5, Tc=5
		qpd = self.qpd
		Ma,Mb,Mc = qpd['Ma'],qpd['Mb'],qpd['Mc']
		Na,Nb,Nc = qpd['Na'],qpd['Nb'],qpd['Nc']
		Ta,Tb,Tc = qpd['Ta'],qpd['Tb'],qpd['Tc']
		nmods = self.numModules
		ratio = (Ta*Ma +Tb*Mb +Tc*Mc)/(Na*Ma +Nb*Mb +Nc*Mc)
		for k in range(self.numModules):
			self.moduleMarks[k] = 60/self.numModules	# Assign equal marks to all Modules
		tot = 0
		self.pm2 = []
		for k in range(self.numModules):
			x = self.moduleMarks[k]*ratio
			self.pm2.append(x)
			tot += x		
			
		ic = []				# Algorithm given by TKM
		for k in range(0,NMODS,2):
			ic.append(randint(0,1)+k)
		while 1:
			n = randint(0,NMODS-1)
			if n not in ic:
				ic.append(n)
				break
		ib = []
		for p in range(NMODS):
			if p not in ic:
				ib.append(p)
		random.shuffle(ib)
		
		ia = ic.copy() 			# Five A types  from the same sections
		ia.append(ib[-1])		# Add third section of B to A	
		ib = (ib*2)[:-1]		# repeat the first two sections of B type 
		return ia,ib,ic	

			
	def selectM70CU(self):   # for CU
		'''
		Ma=3, Mb=6, Mc=10 , Ta=10, Tb=8, Tc=2 and
		Ma=2, Mb=6, Mc=10 , Ta=10, Tb=5, Tc=2 
		'''		
		qpd = self.qpd
		Ma,Mb,Mc = qpd['Ma'],qpd['Mb'],qpd['Mc']
		Na,Nb,Nc = qpd['Na'],qpd['Nb'],qpd['Nc']
		Ta,Tb,Tc = qpd['Ta'],qpd['Tb'],qpd['Tc']
		nmods = self.numModules
		ratio = (Ta*Ma +Tb*Mb +Tc*Mc)/(Na*Ma +Nb*Mb +Nc*Mc)
		tot = 0
		self.pm2 = []
		for k in range(self.numModules):
			x = self.moduleMarks[k]*ratio
			self.pm2.append(x)
			tot += x

		trial = 0
		ic = []			# fill C types, handle sections without C also
		while len(ic) < Tc and trial < 10:			
			n = randint(0,nmods-1)  
			trial += 1
			if n not in ic:
				ic.append(n)
				print(trial)

		ib = []
		for n in range(nmods):	# fill B 1 round(exclude C filled)
			k = n % nmods
			mark = Mb * ib.count(k) + Mc * ic.count(k)
			if Mb+mark <= self.pm2[k] and len(ib) < Tb:   
				ib.append(k)
			else:
				print('B',n,k,'|', mark, '+',Mb,' ',self.moduleMarks[k],'|',len(ib), Tb, ic)
			
		ia = []	
		# fill remaining as per the moduleMarks values
		for n in range(50):		
			k = n%nmods #randint(0,nmods-1)
			mark = Ma * ia.count(k) + Mb * ib.count(k) + Mc * ic.count(k)
			if(Mb+mark <= self.pm2[k]*1.1 and len(ib) < Tb and k not in ic):
				ib.append(k)
			else:
				pass
				#print(n,k,'|', mark, '+',Mb,' ',self.moduleMarks[k],'|',len(ib), Tb, ic)

			mark = Ma * ia.count(k) + Mb * ib.count(k) + Mc * ic.count(k)
			if(Ma+mark <= self.pm2[k]*1.1 and len(ia) < Ta):
				ia.append(k)
			else:
				print(n,k,'|', mark, '+',Ma,' ',self.moduleMarks[k],'|',len(ia), Ta, ic)


			if len(ia) == qpd['Ta'] and len(ib) == qpd['Tb'] and len(ic) == qpd['Tc']:
				break
			
		if len(ia) != qpd['Ta'] or len(ib) != qpd['Tb'] or len(ic) != qpd['Tc']:
			err = 'Failed to fill all %d sections '%nmods
			print(err, self.moduleMarks)
			self.showDist(ia,ib,ic)
			sys.exit(err)
		return ia,ib,ic
			
	def selectM50CU(self):   # Common for several qpModels
		'''
		Ma=3, Mb=6, Mc=10 , Ta=10, Tb=8, Tc=2 and
		Ma=2, Mb=6, Mc=10 , Ta=10, Tb=5, Tc=2 
		'''		
		qpd = self.qpd
		Ma,Mb,Mc = qpd['Ma'],qpd['Mb'],qpd['Mc']
		Na,Nb,Nc = qpd['Na'],qpd['Nb'],qpd['Nc']
		Ta,Tb,Tc = qpd['Ta'],qpd['Tb'],qpd['Tc']
		nmods = self.numModules
		ratio = (Ta*Ma +Tb*Mb +Tc*Mc)/(Na*Ma +Nb*Mb +Nc*Mc)
		tot = 0
		self.pm2 = []
		for k in range(self.numModules):
			x = self.moduleMarks[k]*ratio
			self.pm2.append(x)
			tot += x

		trial = 0
		ic = []			# fill C types, handle sections without C also
		while len(ic) < Tc and trial < 10:			
			n = randint(0,nmods-1)  
			trial += 1
			if n not in ic:
				ic.append(n)
				print(trial)

		ib = []
		for n in range(nmods):	# fill B 1 round(exclude C filled)
			k = n % nmods
			mark = Mb * ib.count(k) + Mc * ic.count(k)
			if Mb+mark <= self.pm2[k] and len(ib) < Tb:   
				ib.append(k)
			else:
				print('B',n,k,'|', mark, '+',Mb,' ',self.moduleMarks[k],'|',len(ib), Tb, ic)
			
		ia = []	
		# fill remaining as per the moduleMarks values
		for n in range(50):		
			k = n%nmods #randint(0,nmods-1)
			mark = Ma * ia.count(k) + Mb * ib.count(k) + Mc * ic.count(k)
			if(Mb+mark <= self.pm2[k]*1.1 and len(ib) < Tb and k not in ic):
				ib.append(k)
			else:
				pass
				#print(n,k,'|', mark, '+',Mb,' ',self.moduleMarks[k],'|',len(ib), Tb, ic)

			mark = Ma * ia.count(k) + Mb * ib.count(k) + Mc * ic.count(k)
			if(Ma+mark <= self.pm2[k]*1.1 and len(ia) < Ta):
				ia.append(k)
			else:
				print(n,k,'|', mark, '+',Ma,' ',self.moduleMarks[k],'|',len(ia), Ta, ic)


			if len(ia) == qpd['Ta'] and len(ib) == qpd['Tb'] and len(ic) == qpd['Tc']:
				break
			
		if len(ia) != qpd['Ta'] or len(ib) != qpd['Tb'] or len(ic) != qpd['Tc']:
			err = 'Failed to fill all %d sections '%nmods
			print(err, self.moduleMarks)
			self.showDist(ia,ib,ic)
			sys.exit(err)
		return ia,ib,ic
			
					
	def makeQP(self):
		if self.qpModel == 'M70CU':
			self.qpd = qpdataM70CU
			ia,ib,ic = self.selectM70CU()				
		elif self.qpModel == 'M50CU':
			self.qpd = qpdataM50CU
			ia,ib,ic = self.selectM50CU()				
		elif self.qpModel == 'M70KU':
			self.qpd = qpdataM70KU
			ia,ib,ic = self.selectM70KU()	
		elif self.qpModel == 'M50KU':
			self.qpd = qpdataM50KU
			ia,ib,ic = self.selectM50KU()	
		elif self.qpModel == 'M60KU':  # KU department only
			self.qpd = qpdataM60KU
			ia,ib,ic = self.selectM60KU()	
		else:	
			sys.exit('Error: Unsupported Question paper type.')	
		ia.sort()		
		ib.sort()

		print ('IABC = ', ia, ib, ic)
		self.showDist(ia, ib, ic)
		
		self.MAXMARK = self.qpd['Ma'] * self.qpd['Na'] + self.qpd['Mb']\
		 * self.qpd['Nb'] +	self.qpd['Mc'] * self.qpd['Nc']
		for k in range(len(ic)):		# Select Ctype first
			module = ic[k]				# Pre-selected module #s are stored in ic
			random.shuffle(self.Ctypes[module])
			for q in self.Ctypes[module]:
				if q[QUSED] == False:
					q[QUSED] = True
					self.CQ.append(q)
					#print('First C: ', module, q[QSN], q[QMARK])
					self.markRelations(module, q[QRELATIONS])
					if	q[QMARK] != self.qpd['Mc']:		# get one more piece 
						foundQ2 = False
						mr2 = self.qpd['Mc'] - q[QMARK]
						print ('CM mr mr2 = ', q[QMARK],mr2)
						random.shuffle(self.Ctypes[module])
						for q2 in self.Ctypes[module]:
							if q2[QMARK] == mr2 and q2[QUSED] == False:
								q2[QUSED] = True
								foundQ2 = True
								self.CQ[-1].append(q2)	# next piece
								#print('Second C: ', module, q2[QSN], q2[QMARK])
								self.markRelations(module, q[QRELATIONS])
								break
						if foundQ2 == False:
							print('Failed to get Second part:', module, q[QSN], q[QMARK],q) 
							sys.exit('Failed to get Second part of M%d:Q%d'%(module,q[QSN]))
					break
					
		for k in range(len(ib)):	# Select Ctype first
			module = ib[k]			
			random.shuffle(self.Btypes[module])		
			for q in self.Btypes[module]:
				if q[QUSED] == False:
					q[QUSED] = True
					self.BQ.append(q)
					#self.markRelations(module, q[QRELATIONS])
					break
		
		for k in range(len(ia)):	# Select A type
			module = ia[k]			
			random.shuffle(self.Atypes[module])
			for q in self.Atypes[module]:
				if(q == None):
					print ('Q is None',self.Atypes[module])
				if q[QUSED] == False:
					q[QUSED] = True
					self.AQ.append(q)
					#self.markRelations(module, q[QRELATIONS])
					break

		#Verify the results.
		if len(self.AQ) != len(ia):
			print('Could get only %d number of A type questions. Requires %d'%(len(self.AQ), len(ia)))
			sys.exit(1)
		if len(self.BQ) != len(ib):
			print('Could get only %d number of B type questions. Requires %d'%(len(self.BQ), len(ib)))
			sys.exit(1)
		if len(self.CQ) != len(ic):
			print('Could get only %d number of C type questions. Requires %d'%(len(self.CQ), len(ic)))
			sys.exit(1)
	
		# End of make QP	
		
	def makeTexFiles(self):
		now = datetime.now() + timedelta(hours = 5, minutes=30)
		self.dt = now.strftime("%d/%m/%Y %H:%M:%S")

		#print('Python: Answer Done. Making QP..')
		#--------Generate Latex document, using data from AQ,BQ and CQ -----
		tex = '\\documentclass[12pt, a4paper]{exam}\n \\usepackage{graphicx}\n \\usepackage{amsmath}\n \\usepackage{amssymb}\n'
		tex += '\\usepackage[left=0.8in, top=0.7in, bottom=0.7in]{geometry}\n'

		if self.language.lower() == 'malayalam':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Malayalam]{Rachana}'			
		elif self.language.lower() == 'hindi':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'
		elif self.language.lower() == 'arabic':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setotherlanguage{arabic}\n'
			tex += '\\setmainfont[Script=Arabic,Scale=1.1]{Scheherazade}\n'	
			tex += '\\usepackage{bidi}\n\\newcommand{\PRL}\n\n' 

		tex += '\\begin{document}\n'
		
		tex += '\\begin{center}\n'
		tex += 'Reg No:.................. \\hfill \\small {%s}\n\n'%(qpCode)
		tex += 'Name :.................... \\hfill  \\small {%s}\n\n'%('')	
		
		tex += '\\large\n \\textbf {%s Semester %s %s Examination\n'%(self.semester, self.degree, self.subject)
		tex += '\\\\%s %s\n'%(examMonth, examYear)
		courseYear = self.courseCode[1:5]
		tex += '(%s Admission onwards)\\\\\n'%courseYear # syllabusYear
		tex += '%s (%s)}\n\n'%(self.courseCode[6:],self.courseName)
		tex += '(%s)\n\n'%qpHeader
		tex += '\\vspace{0.1in}\n\\end{center}\n '
		tex += '\\hrule\n\\vspace{0.1in}\n'
	
		tex += '\\noindent\n' 
		tex += 'Time : %3d min  \hfill Maximum Marks : %2d\n\\vspace{0.1in}\n\n'%(self.qpd['DUR'], self.MAXMARK)
		
		tex += '{\\noindent \\bfseries  Part A (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Na'], self.qpd['Ma'])

		tex += '\\begin{questions} \\pointsdroppedatright\n'

		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.AQ:
			tex += '\\question[%d] %s \\droppoints\n'%(q[QMARK],q[QTEXT].strip())
			tex += '\\vspace{0.05in}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'

		tex += '{\\noindent \\small \\bfseries  Part B (Answer any %d questions. Each carries %d marks) }\n'%(self.qpd['Nb'], self.qpd['Mb'])

		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.BQ:
			tex += '\\question[%d] %s \\droppoints\n'%(q[QMARK],q[QTEXT].strip())
			tex += '\\vspace{0.05in}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'
		
		tex += '{\\noindent\n\\small \\bfseries Part C (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Nc'], self.qpd['Mc'])
		
		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'		
		for q in self.CQ:
			if len(q) == SINGLEQL:
				tex += '\\question[%d] %s \\droppoints\n'%(q[QMARK],q[QTEXT].strip())
				tex += '\\vspace{0.05in}\n'
			else:
				q2 = q[QPART2]
				tex += '\\question\n'
				tex += '\\begin{parts} \n'
				tex += '\\part[%d] %s \\droppoints'%(q[QMARK], q[QTEXT].strip())
				tex += '\\part[%d] %s \\droppoints'%(q2[QMARK], q2[QTEXT].strip())
				tex += '\\end{parts}'	
				tex += '\\vspace{0.05in}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'

		tex += '\\end{questions}\n'
		tex += '\\end{document}'

		fn = courseCode
		open(fn + '.tex', 'w').write(tex)
		#print('Python: Written Question .tex file ',fn)
		self.answerTex()

	def answerTex(self):
		#print('Python: Entering answerTex..')
		#--Generate Latex document, using questions & answers from AQ,BQ and CQ -----
		tex = '\\documentclass[12pt, a4paper]{exam}\n \\usepackage{graphicx}\n \\usepackage{amsmath}\n \\usepackage{amssymb}\n'
		tex += '\\usepackage[left=0.8in, top=0.7in, bottom=0.7in]{geometry}\n'

		if self.language.lower() == 'malayalam':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Malayalam]{Rachana}'			
		elif self.language.lower() == 'hindi':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'		
		elif self.language.lower() == 'arabic':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setotherlanguage{arabic}\n'
			tex += '\\setmainfont[Script=Arabic, Scale=1.1]{Scheherazade}\n'	
			tex += '\\usepackage{bidi}\n\\newcommand{\PRL}\n\n' 

		tex += '\\begin{document} \\noindent\n'

		tex += '%s \\hfill %s\n\\newline '%(qpCode, self.dt)
		
		tex += '\\begin{center} \\large\n \\textbf {%s Semester %s %s Examination\n'%(self.semester, self.degree, self.subject)
		tex += '%s %s\n'%(examMonth, examYear)
		tex += '(%s Admission onwards)\\\\\n'%self.syllabusYear
		tex += '%s (%s)}\n\\end{center}\n'%(self.courseCode[6:], self.courseName)
		
		tex += '\\noindent\n' 
		tex += 'Time : %s  \hfill Maximum Marks : %2d\n\\vspace{0.1in}\n\n'%(self.qpd['DUR'], self.MAXMARK)
		tex += '\\hrule\n\\vspace{0.1in}\n'

		tex += '{\\noindent \\bfseries  Part A (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Na'], self.qpd['Ma'])
		tex += '\\begin{questions} \\pointsdroppedatright\n'

		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.AQ:
			tex += '\\question[%d]M%d(%d):%s\n\n'%(q[QMARK],q[QMODNUM],q[QSN],q[QTEXT].strip())
			
			if self.language.lower() == 'arabic':
				tex += '\\textbf{إجابة}\n\n'
			else:
				tex += '\\textbf{Answer:}\n\n'
			
			tex += '%s\n\\droppoints\n'%q[QATEXT].strip()
			tex += '\\vspace{0.05in}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'

		tex += '{\\noindent \\small \\bfseries  Part B (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Nb'], self.qpd['Mb'])

		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.BQ:
			tex += '\\question[%d]M%d(%d):%s\n\n'%(q[QMARK],q[QMODNUM],q[QSN],q[QTEXT].strip())
			
			if self.language.lower() == 'arabic':
				tex += '\\textbf{إجابة}\n\n'
			else:
				tex += '\\textbf{Answer:}\n\n'
			
			tex += '%s\n\\droppoints\n'%q[QATEXT].strip()
			tex += '\\vspace{0.05in}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'
			
		tex += '{\\noindent\n\\small \\bfseries Part C (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Nc'], self.qpd['Mc'])

		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.CQ:
			if len(q) == SINGLEQL:
				tex += '\\question[%d]M%d(%d):%s\n\n'%(q[QMARK],q[QMODNUM],q[QSN],q[QTEXT].strip())
				
				if self.language.lower() == 'arabic':
					tex += '\\textbf{إجابة}\n\n'
				else:
					tex += '\\textbf{Answer:}\n\n'
				
				tex += '%s\n\\droppoints\n'%q[QATEXT].strip()
				tex += '\\vspace{0.05in}\n'
			else:
				q2 = q[QPART2]
				tex += '\\question\n'
				tex += '\\begin{parts} \n'
				tex +='\\part[%d]M%d(%d):%s\n\n'%(q[QMARK],q[QMODNUM],q[QSN],q[QTEXT].strip())

				if self.language.lower() == 'arabic':
					tex += '\\textbf{إجابة}\n\n'
				else:
					tex += '\\textbf{Answer:}\n\n'

				tex += '%s\\droppoints\n'%q[QATEXT].strip()				
				tex += '\\part[%d]M%d(%d): %s\n\n'%(q2[QMARK],q2[QMODNUM],q2[QSN],q2[QTEXT].strip())
				
				if self.language.lower() == 'arabic':
					tex += '\\textbf{إجابة}\n\n'
				else:
					tex += '\\textbf{Answer:}\n\n'

				tex += '%s\\droppoints\n'%q2[QATEXT].strip()
				tex += '\\end{parts}'	
				tex += '\\vspace{0.05in}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'

		tex += '\\end{questions}\n'
		tex += '\\end{document}'

		fn = courseCode + '_ANS'
		open(fn + '.tex', 'w').write(tex)
		#print('Python: Written Answer .tex file ',fn)


#---- main program -----

ss = open('info.txt').read().split('\n')
#os.unlink('info.txt')
for row in ss:
	sss = row.split(':')
	#print(sss)
	if sss[0] == 'examYear':
		examYear = sss[1]
	elif sss[0] == 'examMonth':
		examMonth = sss[1]
	elif sss[0] == 'courseCode':
		courseCode = sss[1].strip().split('|')[0].strip()
	elif sss[0] == 'passwd':
		passwd = sss[1]
	elif sss[0] == 'qpCode':
		qpCode = sss[1]
	elif sss[0] == 'header':
		qpHeader = sss[1]
	elif sss[0] == 'level':
		qpLevel = sss[1]

if qpHeader == '': qpHeader = '---'
if qpCode == '': qpCode = '---'
if examYear == '': examYear = '---'
if examMonth == '':  examMonth = '---'

#print(courseCode, passwd, examYear, examMonth, qpCode, qpHeader, qpLevel)

cc = Course()
cc.findCourse(courseCode)
cc.fetchData()	
cc.makeQP()
cc.makeTexFiles()

print('Done')
sys.exit('Done')

